#if !defined(BOOST_FILE_SYSTEM_H_INCLUDED_)
#define BOOST_FILE_SYSTEM_H_INCLUDED_

#pragma once

#include	<UndefineMdlMacrosThatInterfereWithBoost.h>
#include 	<boost/filesystem/operations.hpp>

#include	<DgnPlatform\DgnFile.h>
#include	<string>
#include	<tstring_typedef.h>
#define BENTLEY
#include	<std_collection_typedefs.h>
#include	<windows.h>
#include	<Shellapi.h>
///////////////////////////////////////////////////////////////////////
typedef 	std::vector<boost::filesystem::path>	PathCollection;
typedef 	PathCollection::iterator				PathIterator;
typedef 	PathCollection::const_iterator			PathConstIterator;
///////////////////////////////////////////////////////////////////////
///	<summary>FileCollector harvests files in a root directory.
///	<para>If recursing is required, descends into all folders beneath the root.</para>
///	</summary>
class FileCollector
{
	bool						recurse_;
	bool						cad_;
	bool						writable_;
	PathCollection				paths_;
	WideStringCollection		files_;
	std::wstring				root_;

	size_t						HarvestRecursive	(boost::filesystem::path const& p);
	size_t						HarvestRootOnly		(boost::filesystem::path const& p);
	///	<summary>If the path refers to a file, add it to our list.</summary>
	bool						InsertPath			(boost::filesystem::path const& p);

public:
	enum	CollectFileFlags
	{
		//	Mode
		WriteableFiles		= 0x0001 << 0,
		ReadOnlyFiles		= 0x0001 << 1,
		ReadWriteFiles		= WriteableFiles | ReadOnlyFiles,
		ExecutableFiles		= 0x0001 << 2,
		Folders				= 0x0001 << 3,
		Links				= 0x0001 << 4,
		//	Type
		DgnV8Files			= 0x0001 << 8,
		DgnV7Files			= 0x0001 << 9,
		DgnFiles			= (DgnV8Files | DgnV7Files),
		DwgFiles			= 0x0001 << 10,
		DxfFiles			= 0x0001 << 11,
		AutoCadFiles		= (DwgFiles | DxfFiles),
		CadFiles			= (DgnFiles | AutoCadFiles),
		//	Source
		FromFolder			= 0x0001 << 16,
		FromDgnDir			= 0x0001 << 17,
		FromProject			= 0x0001 << 18,
		//	Convenient bundles
		WriteableCadFiles	= WriteableFiles | DgnV8Files | DwgFiles,
		ReadableDgnFiles	= WriteableFiles | ReadOnlyFiles | DgnFiles | DwgFiles,
	};
	///////////////////////////////////////////////////////////////////
	//	Implementation
	///////////////////////////////////////////////////////////////////
	///	<summary>Get the Windows description of a file type.
	///	<remarks><para>There is one aspect of SHGetFileInfo() that you should be aware of.
	///	Specifically, SHGetFileInfo() takes a relatively long time to execute. </para>
	///	See:
	///	<list>
	///	<item><term>http://www.jasinskionline.com/windowsapi/ref/s/shgetfileinfo.html</term></item>
	///	<item><term>http://bcbjournal.org/articles/vol4/0002/Getting_shell_item_information.htm</term></item>
	///	</list>
	///	</remarks>
	///	</summary>
	static std::wstring			DescribeFileType	(boost::filesystem::path const&	p);
	///	<summary>Removes file extension from a file path.</summary>
	static void					RemoveExtension		(boost::filesystem::path&		p);
	static std::wstring			GetProjectRoot		();
	void						Trace				()	const;
	void						Root				(std::wstring const&	root)	{	root_	= root;	}
	std::wstring const&			Root				()	const	{	return root_;			}
	void						Recurse				(bool r)	{	recurse_	= r;		}
	bool						Recurse				()	const	{	return recurse_;		}
	void						CAD					(bool b)	{	cad_		= b;		}
	bool						CAD					()	const	{ 	return cad_;			}

	WideStringCollection const&	Files				()	const	{	return files_;			}
	PathCollection const&		Paths				()	const	{	return paths_;			}
	size_t						Harvest				(std::wstring const&	root,
													bool					subFolders,
													bool					cadFiles,
													bool					writable);
	size_t						Harvest				(WChar const*			root,
													bool					subFolders,
													bool					cadFiles,
													bool					writable);
	size_t						Harvest				();
	size_t						FileCount			()	const	{	return files_.size ();	}
	///	<summary>
	///	Get the Windows Explorer description of this file's type
	///	</summary>
	tstring						GetFileTypeDescr	(const tstring&		file)	const
	{
		return GetFileTypeDescr (file.c_str ());
	}
	tstring						GetFileTypeDescr	(LPCTSTR			file)	const;
	///	<summary>
	///	Get the Windows Explorer description of this file's type
	/// with additional MicroStation-specific detail.
	///	Dimension will be 2 or 3 for a valid model and 0 otherwise
	///	</summary>
	tstring						GetFileTypeInfo		(const tstring&		file,
													Bentley::DgnPlatform::DgnFileFormatType&	format,
													int&				dimension)	const
	{
		return GetFileTypeInfo (file.c_str (), format, dimension);
	}
	tstring						GetFileTypeInfo		(LPCWSTR			file,
													Bentley::DgnPlatform::DgnFileFormatType&	format,
													int&				dimension)	const;
	///	<summary>
	///	Get the Windows Explorer description of this file's mode (Read-only, link, etc).
	///	<remarks>boost::filesystem can replace most of the ::SHxxx tests.<para>
	///	However, it can't obtain the Windows file type description.</para>
	///	</remarks>
	///	</summary>
	tstring						GetFileMode			(const tstring&		file)	const
	{
		return GetFileMode (file.c_str ());
	}
	tstring						GetFileMode			(LPCTSTR			file)	const;
	///	<summary>	Return true if file can be opened by MicroStation.</summary>
	bool						IsCadFile			(boost::filesystem::path const& 		p)	const;
	///////////////////////////////////////////////////////////////////
	//	Construction
	///	<summary>Default constructor action is to harvest any file type
	//	found in sub-folders that is read or write.</summary>
	FileCollector ()	: recurse_	(true), cad_ (false), writable_ (false)	{}

};

#endif // !defined(BOOST_FILE_SYSTEM_H_INCLUDED_)
